@@ -23,11 +23,13 @@ import random
 import syslog
 
 import pytest
-from mock import sentinel
+import mock
 
 from pyudev import udev_version
-from pyudev._libudev import libudev
 
+from tests._constants import _UDEV_TEST
+
+from tests.utils import is_unicode_string
 
 def test_udev_version():
     assert isinstance(udev_version(), int)
@@ -40,16 +42,16 @@ def test_udev_version():
 class TestContext(object):
 
     def test_sys_path(self, context):
-        assert pytest.is_unicode_string(context.sys_path)
+        assert is_unicode_string(context.sys_path)
         assert context.sys_path == '/sys'
 
     def test_device_path(self, context):
-        assert pytest.is_unicode_string(context.device_path)
+        assert is_unicode_string(context.device_path)
         assert context.device_path == '/dev'
 
-    @pytest.mark.udev_version('>= 167')
+    @_UDEV_TEST(167, "test_run_path")
     def test_run_path(self, context):
-        assert pytest.is_unicode_string(context.run_path)
+        assert is_unicode_string(context.run_path)
         assert context.run_path == '/run/udev'
 
     def test_log_priority_get(self, context):
@@ -57,15 +59,21 @@ class TestContext(object):
         assert syslog.LOG_EMERG <= context.log_priority <= syslog.LOG_DEBUG
 
     def test_log_priority_get_mock(self, context):
-        calls = {'udev_get_log_priority': [(context,)]}
-        with pytest.calls_to_libudev(calls):
-            libudev.udev_get_log_priority.return_value = sentinel.log_priority
-            assert context.log_priority is sentinel.log_priority
+        spec = lambda c: None
+        funcname = 'udev_get_log_priority'
+        with mock.patch.object(context._libudev, funcname,
+                               autospec=spec) as func:
+            func.return_value = mock.sentinel.log_priority
+            assert context.log_priority is mock.sentinel.log_priority
+            func.assert_called_once_with(context)
 
     def test_log_priority_set_mock(self, context):
-        calls = {'udev_set_log_priority': [(context, sentinel.log_priority)]}
-        with pytest.calls_to_libudev(calls):
-            context.log_priority = sentinel.log_priority
+        spec = lambda c, p: None
+        funcname = 'udev_set_log_priority'
+        with mock.patch.object(context._libudev, funcname,
+                               autospec=spec) as func:
+            context.log_priority = mock.sentinel.log_priority
+            func.assert_called_once_with(context, mock.sentinel.log_priority)
 
     def test_log_priority_roundtrip(self, context):
         # FIXME: This adds UDEV_LOG properties?!
