@@ -22,10 +22,20 @@ from __future__ import (print_function, division, unicode_literals,
 import pytest
 import mock
 
-from pyudev import Enumerator, Device
+from hypothesis import given
+from hypothesis import settings
+from hypothesis import strategies
+
+from pyudev import Enumerator
+
+from ._constants import _CONTEXT_STRATEGY
+from ._constants import _DEVICES
+from ._constants import _UDEV_TEST
+from ._constants import _UDEV_VERSION
 
 
-def pytest_funcarg__enumerator(request):
+@pytest.fixture
+def enumerator(request):
     context = request.getfuncargvalue('context')
     return context.list_devices()
 
@@ -40,6 +50,7 @@ class TestEnumerator(object):
     def test_match_subsystem_nomatch(self, context):
         devices = context.list_devices().match_subsystem('input', nomatch=True)
         for device in devices:
+            assert device.subsystem is not None
             assert device.subsystem != 'input'
 
     def test_match_subsystem_nomatch_unfulfillable(self, context):
@@ -48,6 +59,44 @@ class TestEnumerator(object):
         devices.match_subsystem('input', nomatch=True)
         assert not list(devices)
 
+    def test_match_subsystem_nomatch_complete(self, context):
+        """
+        Test that w/ respect to the universe of devices returned by
+        list_devices() a match and its inverse are complements of each other.
+
+        Note that list_devices() omits devices that have no subsystem,
+        so with respect to the whole universe of devices, the two are
+        not complements of each other.
+        """
+        m_devices = set(context.list_devices().match_subsystem('j'))
+        nm_devices = set(
+           context.list_devices().match_subsystem('j', nomatch=True)
+        )
+
+        assert not m_devices.intersection(nm_devices)
+
+        devices = set(context.list_devices())
+        assert devices == m_devices.union(nm_devices)
+
+    def test_match_scsi_devices_children(self, context):
+        """
+        Test that every device with type scsi_device has only one disk
+        descendant.
+        """
+        scsi_devices = context.list_devices().match_property(
+           'DEVTYPE',
+           'scsi_device'
+        )
+
+        def func(dev):
+            return context.list_devices(
+               subsystem='block',
+               DEVTYPE='disk',
+               parent=dev
+            )
+
+        assert all(len(list(func(d))) in (0, 1) for d in scsi_devices)
+
     def test_match_sys_name(self, context):
         devices = context.list_devices().match_sys_name('sda')
         for device in devices:
@@ -74,10 +123,16 @@ class TestEnumerator(object):
             assert device.asbool('ID_INPUT_KEY')
 
     def test_match_attribute_nomatch(self, context):
+        key = 'driver'
+        value = 'usb'
         devices = context.list_devices().match_attribute(
-            'driver', 'usb', nomatch=True)
+           key,
+           value,
+           nomatch=True
+        )
         for device in devices:
-            assert device.attributes.get('driver') != 'usb'
+            attributes = device.attributes
+            assert attributes.get(key) != value
 
     def test_match_attribute_nomatch_unfulfillable(self, context):
         devices = context.list_devices()
@@ -88,7 +143,7 @@ class TestEnumerator(object):
     def test_match_attribute_string(self, context):
         devices = list(context.list_devices().match_attribute('driver', 'usb'))
         for device in devices:
-            assert device.attributes['driver'] == b'usb'
+            assert device.attributes.get('driver') == b'usb'
 
     def test_match_attribute_int(self, context):
         # busnum gives us the number of a USB bus.  And any decent system
@@ -99,7 +154,7 @@ class TestEnumerator(object):
         # any device at all on the system running the test
         devices = list(context.list_devices().match_attribute('busnum', 2))
         for device in devices:
-            assert device.attributes['busnum'] == b'2'
+            assert device.attributes.get('busnum') == b'2'
             assert device.attributes.asint('busnum') == 2
 
     def test_match_attribute_bool(self, context):
@@ -108,41 +163,60 @@ class TestEnumerator(object):
         # test should work on all systems these tests are ever run on
         devices = list(context.list_devices().match_attribute('ro', False))
         for device in devices:
-            assert device.attributes['ro'] == b'0'
+            assert device.attributes.get('ro') == b'0'
             assert not device.attributes.asbool('ro')
 
-    @pytest.mark.udev_version('>= 154')
+    @_UDEV_TEST(154, "test_match_tag_mock")
     def test_match_tag_mock(self, context):
         enumerator = context.list_devices()
-        calls = {'udev_enumerate_add_match_tag': [(enumerator, b'spam')]}
-        with pytest.calls_to_libudev(calls):
+        funcname = 'udev_enumerate_add_match_tag'
+        spec = lambda e, t: None
+        with mock.patch.object(enumerator._libudev, funcname,
+                               autospec=spec) as func:
             retval = enumerator.match_tag('spam')
             assert retval is enumerator
+            func.assert_called_with(enumerator, b'spam')
 
-    @pytest.mark.udev_version('>= 154')
+    @_UDEV_TEST(154, "test_match_tag")
     def test_match_tag(self, context):
         devices = list(context.list_devices().match_tag('seat'))
         for device in devices:
             assert 'seat' in device.tags
 
-    @pytest.mark.parametrize('device_data', pytest.config.udev_device_sample)
-    @pytest.mark.udev_version('>= 172')
-    def test_match_parent(self, context, device_data):
-        device = Device.from_path(context, device_data.device_path)
-        parent = device.parent
-        if parent is None:
-            pytest.skip('Device {0!r} has no parent'.format(device))
-        else:
-            children = list(context.list_devices().match_parent(parent))
-            assert device in children
-            assert parent in children
+    _devices = [d for d in _DEVICES if d.parent]
+    @pytest.mark.skipif(len(_devices) == 0, reason="no device with parent")
+    @given(_CONTEXT_STRATEGY, strategies.sampled_from(_devices))
+    @settings(max_examples=5, min_satisfying_examples=1)
+    def test_match_parent(self, context, device):
+        """
+        For a given device, verify that it is in its parent's children.
 
-    @pytest.mark.udev_version('>= 165')
+        Verify that the parent is also among the device's children,
+        unless the parent lacks a subsystem
+
+        See: rhbz#1255191
+        """
+        parent = device.parent
+        children = list(context.list_devices().match_parent(parent))
+        assert device in children
+        if _UDEV_VERSION <= 175:
+            assert parent in children
+        else:
+            if parent.subsystem is not None:
+                assert parent in children
+            else:
+                assert parent not in children
+
+    @_UDEV_TEST(165, "test_match_is_initialized_mock")
     def test_match_is_initialized_mock(self, context):
         enumerator = context.list_devices()
-        calls = {'udev_enumerate_add_match_is_initialized': [(enumerator,)]}
-        with pytest.calls_to_libudev(calls):
-            enumerator.match_is_initialized()
+        funcname = 'udev_enumerate_add_match_is_initialized'
+        spec = lambda e: None
+        with mock.patch.object(enumerator._libudev, funcname,
+                               autospec=spec) as func:
+            retval = enumerator.match_is_initialized()
+            assert retval is enumerator
+            func.assert_called_with(enumerator)
 
     def test_combined_matches_of_same_type(self, context):
         """
@@ -190,7 +264,7 @@ class TestEnumerator(object):
             enumerator.match(tag=mock.sentinel.tag)
             match_tag.assert_called_with(mock.sentinel.tag)
 
-    @pytest.mark.udev_version('>= 172')
+    @_UDEV_TEST(172, "test_match_passthrough_parent")
     def test_match_passthrough_parent(self, enumerator):
         with mock.patch.object(enumerator, 'match_parent',
                                autospec=True) as match_parent:
