@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2010, 2011, 2012 Sebastian Wiesner <lunaryorn@gmail.com>
+# Copyright (C) 2010, 2011, 2012, 2013 Sebastian Wiesner <lunaryorn@gmail.com>
 
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -19,19 +19,23 @@ from __future__ import (print_function, division, unicode_literals,
                         absolute_import)
 
 import pytest
-from mock import Mock
+import mock
 
 from pyudev import Monitor, Device
-from pyudev._libudev import libudev
 
+from tests.utils.udev import DeviceDatabase
+from tests.utils.udev import get_device_sample
 
-def pytest_funcarg__monitor(request):
+@pytest.fixture
+def monitor(request):
     return Monitor.from_netlink(request.getfuncargvalue('context'))
 
 
-def pytest_funcarg__fake_monitor_device(request):
+@pytest.fixture
+def fake_monitor_device(request):
     context = request.getfuncargvalue('context')
-    return Device.from_path(context, '/devices/platform')
+    device = get_device_sample(DeviceDatabase.db(), sample_size=1)[0]
+    return Device.from_path(context, device.device_path)
 
 
 def test_fake_monitor(fake_monitor, fake_monitor_device):
@@ -44,9 +48,6 @@ def test_fake_monitor(fake_monitor, fake_monitor_device):
     assert device == fake_monitor_device
 
 
-ACTIONS = ('add', 'remove', 'change', 'move')
-
-
 class ObserverTestBase(object):
 
     def setup_method(self, method):
@@ -83,11 +84,6 @@ class ObserverTestBase(object):
     def connect_signal(self, callback, action=None):
         raise NotImplementedError()
 
-    def stop_when_done(self, *args, **kwargs):
-        self.no_emitted_signals += 1
-        if self.no_emitted_signals >= 2:
-            self.stop_event_loop()
-
     def prepare_test(self, monitor):
         self.create_event_loop(self_stop_timeout=5000)
         self.create_observer(monitor)
@@ -97,21 +93,14 @@ class ObserverTestBase(object):
         # test that the monitor attribute is correct
         assert self.observer.monitor is fake_monitor
 
-    @pytest.mark.parametrize('action', ACTIONS, ids=ACTIONS)
-    def test_events_fake_monitor(self, action, fake_monitor,
-                                 fake_monitor_device):
+    def test_events_fake_monitor(self, fake_monitor, fake_monitor_device):
         self.prepare_test(fake_monitor)
-        event_callback = Mock(side_effect=self.stop_when_done)
-        action_callback = Mock(side_effect=self.stop_when_done)
+        event_callback = mock.Mock(
+            side_effect=lambda *args: self.stop_event_loop())
         self.connect_signal(event_callback)
-        self.connect_signal(action_callback, action=action)
-        calls = {'udev_device_get_action': [(fake_monitor_device,),
-                                            (fake_monitor_device,)]}
-        with pytest.calls_to_libudev(calls):
-            libudev.udev_device_get_action.return_value = action.encode('ascii')
-            self.start_event_loop(fake_monitor.trigger_event)
-        event_callback.assert_called_with(action, fake_monitor_device)
-        action_callback.assert_called_with(fake_monitor_device)
+
+        self.start_event_loop(fake_monitor.trigger_event)
+        event_callback.assert_called_with(fake_monitor_device)
 
     @pytest.mark.privileged
     def test_events_real(self, context, monitor):
@@ -121,38 +110,23 @@ class ObserverTestBase(object):
         monitor.start()
         self.prepare_test(monitor)
         # setup signal handlers
-        event_callback = Mock(side_effect=self.stop_when_done)
-        added_callback = Mock(side_effect=self.stop_when_done)
-        removed_callback = Mock(side_effect=self.stop_when_done)
+        event_callback = mock.Mock(
+            side_effect=lambda *args: self.stop_event_loop())
         self.connect_signal(event_callback)
-        self.connect_signal(added_callback, action='add')
-        self.connect_signal(removed_callback, action='remove')
 
         # test add event
         self.start_event_loop(pytest.load_dummy)
         device = Device.from_path(context, '/devices/virtual/net/dummy0')
-        event_callback.assert_called_with('add', device)
-        added_callback.assert_called_with(device)
-        assert not removed_callback.called
+        event_callback.assert_called_with(device)
 
-        for mock in (event_callback, added_callback, removed_callback):
-            mock.reset_mock()
+        event_callback.reset_mock()
 
         self.start_event_loop(pytest.unload_dummy)
-        event_callback.assert_called_with('remove', device)
-        assert not added_callback.called
-        removed_callback.assert_called_with(device)
+        event_callback.assert_called_with(device)
 
 
 class QtObserverTestBase(ObserverTestBase):
 
-    ACTION_SIGNAL_MAP = {
-        'add': 'deviceAdded',
-        'remove': 'deviceRemoved',
-        'change': 'deviceChanged',
-        'move': 'deviceMoved',
-    }
-
     def setup(self):
         self.qtcore = pytest.importorskip('{0}.QtCore'.format(
             self.BINDING_NAME))
@@ -160,14 +134,10 @@ class QtObserverTestBase(ObserverTestBase):
     def create_observer(self, monitor):
         name = self.BINDING_NAME.lower()
         mod = __import__('pyudev.{0}'.format(name), None, None, [name])
-        self.observer = mod.QUDevMonitorObserver(monitor)
+        self.observer = mod.MonitorObserver(monitor)
 
-    def connect_signal(self, callback, action=None):
-        if action is None:
-            self.observer.deviceEvent.connect(callback)
-        else:
-            signal = getattr(self.observer, self.ACTION_SIGNAL_MAP[action])
-            signal.connect(callback)
+    def connect_signal(self, callback):
+        self.observer.deviceEvent.connect(callback)
 
     def create_event_loop(self, self_stop_timeout=5000):
         self.app = self.qtcore.QCoreApplication.instance()
@@ -192,14 +162,10 @@ class TestPyQt4Observer(QtObserverTestBase):
     BINDING_NAME = 'PyQt4'
 
 
-class TestGlibObserver(ObserverTestBase):
+class TestPyQt5Observer(QtObserverTestBase):
+    BINDING_NAME = 'PyQt5'
 
-    ACTION_SIGNAL_MAP = {
-        'add': 'device-added',
-        'remove': 'device-removed',
-        'change': 'device-changed',
-        'move': 'device-moved',
-    }
+class TestGlibObserver(ObserverTestBase):
 
     def setup(self):
         self.event_sources = []
@@ -212,17 +178,15 @@ class TestGlibObserver(ObserverTestBase):
             self.glib.source_remove(source)
 
     def create_observer(self, monitor):
-        from pyudev.glib import GUDevMonitorObserver
-        self.observer = GUDevMonitorObserver(monitor)
+        from pyudev.glib import MonitorObserver
+        self.observer = MonitorObserver(monitor)
 
-    def connect_signal(self, callback, action=None):
+    def connect_signal(self, callback):
         # drop the sender argument from glib signal connections
         def _wrapper(obj, *args, **kwargs):
             return callback(*args, **kwargs)
-        if action is None:
-            self.observer.connect('device-event', _wrapper)
-        else:
-            self.observer.connect(self.ACTION_SIGNAL_MAP[action], _wrapper)
+
+        self.observer.connect('device-event', _wrapper)
 
     def create_event_loop(self, self_stop_timeout=5000):
         self.mainloop = self.glib.MainLoop()
@@ -250,24 +214,15 @@ class TestWxObserver(ObserverTestBase):
 
     def create_observer(self, monitor):
         from pyudev import wx
-        self.observer = wx.WxUDevMonitorObserver(monitor)
-        self.action_event_map = {
-                'add': wx.EVT_DEVICE_ADDED,
-                'remove': wx.EVT_DEVICE_REMOVED,
-                'change': wx.EVT_DEVICE_CHANGED,
-                'move': wx.EVT_DEVICE_MOVED
-        }
+        self.observer = wx.MonitorObserver(monitor)
 
-    def connect_signal(self, callback, action=None):
-        if action is None:
-            from pyudev.wx import EVT_DEVICE_EVENT
-            def _wrapper(event):
-                return callback(event.action, event.device)
-            self.observer.Bind(EVT_DEVICE_EVENT, _wrapper)
-        else:
-            def _wrapper(event):
-                return callback(event.device)
-            self.observer.Bind(self.action_event_map[action], _wrapper)
+    def connect_signal(self, callback):
+        from pyudev.wx import EVT_DEVICE_EVENT
+
+        def _wrapper(event):
+            return callback(event.device)
+
+        self.observer.Bind(EVT_DEVICE_EVENT, _wrapper)
 
     def create_event_loop(self, self_stop_timeout=5000):
         self.app = self.wx.App(False)
