@@ -22,8 +22,18 @@ import sys
 import errno
 
 import pytest
+from mock import Mock
+
+from hypothesis import given
+from hypothesis import settings
+from hypothesis import strategies
 
 from pyudev import _util
+from pyudev import Context
+
+from .utils import is_unicode_string
+
+_CONTEXT = Context()
 
 
 @pytest.mark.conversion
@@ -42,7 +52,7 @@ def test_ensure_byte_string_none():
 
 @pytest.mark.conversion
 def test_ensure_unicode_string():
-    assert pytest.is_unicode_string(
+    assert is_unicode_string(
         _util.ensure_unicode_string(b'hello world'))
     assert _util.ensure_unicode_string(b'hello world') == 'hello world'
     hello = 'hello world'
@@ -97,15 +107,15 @@ def test_string_to_bool_invalid_value():
 
 
 def test_udev_list_iterate_no_entry():
-    assert not list(_util.udev_list_iterate(None))
+    assert not list(_util.udev_list_iterate(Mock(), None))
 
 
 def test_udev_list_iterate_mock():
-    from pyudev._libudev import libudev
+    libudev = Mock(name='libudev')
     items = [('spam', 'eggs'), ('foo', 'bar')]
-    with pytest.libudev_list('udev_enumerate_get_list_entry', items):
+    with pytest.libudev_list(libudev, 'udev_enumerate_get_list_entry', items):
         udev_list = libudev.udev_enumerate_get_list_entry()
-        assert list(_util.udev_list_iterate(udev_list)) == [
+        assert list(_util.udev_list_iterate(libudev, udev_list)) == [
             ('spam', 'eggs'), ('foo', 'bar')]
 
 
@@ -113,13 +123,25 @@ def raise_valueerror():
     raise ValueError('from function')
 
 
-def test_get_device_type_character_device():
-    assert _util.get_device_type('/dev/console') == 'char'
-
-
-def test_get_device_type_block_device():
-    assert _util.get_device_type('/dev/sda') == 'block'
+_char_devices = list(_CONTEXT.list_devices(subsystem="tty"))
+@pytest.mark.skipif(len(_char_devices) == 0, reason='no tty devices')
+@given(strategies.sampled_from(_char_devices))
+@settings(min_satisfying_examples=1, max_examples=5)
+def test_get_device_type_character_device(a_device):
+    """
+    Check that the device type of a character device is actually char.
+    """
+    assert _util.get_device_type(a_device.device_node) == 'char'
 
+_block_devices = list(_CONTEXT.list_devices(subsystem="block"))
+@pytest.mark.skipif(len(_block_devices) == 0, reason='no block devices')
+@given(strategies.sampled_from(_block_devices))
+@settings(min_satisfying_examples=1, max_examples=5)
+def test_get_device_type_block_device(a_device):
+    """
+    Check that the device type of a block device is actually block.
+    """
+    assert _util.get_device_type(a_device.device_node) == 'block'
 
 def test_get_device_type_no_device_file(tmpdir):
     filename = tmpdir.join('test')
@@ -131,8 +153,41 @@ def test_get_device_type_no_device_file(tmpdir):
 
 
 def test_get_device_type_not_existing(tmpdir):
-    filename = tmpdir.join('test')
+    """
+    Test that an OSError is raised when checking device type using a file
+    that does not actually exist.
+    """
+    filename = tmpdir.join('test_get_device_type_not_existing')
     assert not tmpdir.check(file=True)
-    with pytest.raises(EnvironmentError) as excinfo:
+    with pytest.raises(OSError):
         _util.get_device_type(str(filename))
-    pytest.assert_env_error(excinfo.value, errno.ENOENT, str(filename))
+
+
+def test_eintr_retry_call(tmpdir):
+    import os, signal, select
+
+    def handle_alarm(signum, frame):
+        pass
+    orig_alarm = signal.getsignal(signal.SIGALRM)
+
+    # Open an empty file and use it to wait for exceptions which should
+    # never happen
+    filename = tmpdir.join('test')
+    filename.ensure(file=True)
+    fd = os.open(str(filename), os.O_RDONLY)
+
+    try:
+        signal.signal(signal.SIGALRM, handle_alarm)
+
+        # Ensure that a signal raises EINTR on Python < 3.5
+        if sys.version_info < (3, 5):
+            with pytest.raises(select.error) as e:
+                signal.alarm(1)
+                select.select([], [], [fd], 2)
+
+        # Ensure that wrapping the call does not raise EINTR
+        signal.alarm(1)
+        assert _util.eintr_retry_call(select.select, [], [], [3], 2) == ([], [], [])
+    finally:
+        os.close(fd)
+        signal.signal(signal.SIGALRM, orig_alarm)
